@@ -18,16 +18,23 @@ Weights and Biases Logger
 import os
 from argparse import Namespace
 from pathlib import Path
-from typing import Any, Callable, Dict, List, Mapping, Optional, Sequence, Union
-from weakref import ReferenceType
+from typing import Any, Dict, List, Mapping, Optional, Union
 
 import torch.nn as nn
+from lightning_utilities.core.imports import RequirementCache
+from torch import Tensor
 
+from lightning_lite.utilities.types import _PATH
 from pytorch_lightning.callbacks import Checkpoint
 from pytorch_lightning.loggers.logger import Logger, rank_zero_experiment
 from pytorch_lightning.utilities.exceptions import MisconfigurationException
-from pytorch_lightning.utilities.imports import _RequirementAvailable
-from pytorch_lightning.utilities.logger import _add_prefix, _convert_params, _flatten_dict, _sanitize_callable_params
+from pytorch_lightning.utilities.logger import (
+    _add_prefix,
+    _convert_params,
+    _flatten_dict,
+    _sanitize_callable_params,
+    _scan_checkpoints,
+)
 from pytorch_lightning.utilities.rank_zero import rank_zero_only, rank_zero_warn
 
 try:
@@ -36,11 +43,11 @@ try:
     from wandb.wandb_run import Run
 except ModuleNotFoundError:
     # needed for test mocks, these tests shall be updated
-    wandb, Run, RunDisabled = None, None, None  # type: ignore
+    wandb, Run, RunDisabled = None, None, None
 
-_WANDB_AVAILABLE = _RequirementAvailable("wandb")
-_WANDB_GREATER_EQUAL_0_10_22 = _RequirementAvailable("wandb>=0.10.22")
-_WANDB_GREATER_EQUAL_0_12_10 = _RequirementAvailable("wandb>=0.12.10")
+_WANDB_AVAILABLE = RequirementCache("wandb")
+_WANDB_GREATER_EQUAL_0_10_22 = RequirementCache("wandb>=0.10.22")
+_WANDB_GREATER_EQUAL_0_12_10 = RequirementCache("wandb>=0.12.10")
 
 
 class WandbLogger(Logger):
@@ -133,7 +140,7 @@ class WandbLogger(Logger):
 
     .. code-block:: python
 
-        wandb_logger.unwatch(model)
+        wandb_logger.experiment.unwatch(model)
 
     **Log model checkpoints**
 
@@ -217,6 +224,36 @@ class WandbLogger(Logger):
         data = [["cheese", wandb.Image(img_1), wandb.Audio(snd_1)], ["wine", wandb.Image(img_2), wandb.Audio(snd_2)]]
         wandb_logger.log_table(key="samples", columns=columns, data=data)
 
+
+    **Downloading and Using Artifacts**
+
+    To download an artifact without starting a run, call the ``download_artifact``
+    function on the class:
+
+    .. code-block:: python
+
+        from pytorch_lightning.loggers import WandbLogger
+
+        artifact_dir = WandbLogger.download_artifact(artifact="path/to/artifact")
+
+    To download an artifact and link it to an ongoing run call the ``download_artifact``
+    function on the logger instance:
+
+    .. code-block:: python
+
+        class MyModule(LightningModule):
+            def any_lightning_module_function_or_hook(self):
+                self.logger.download_artifact(artifact="path/to/artifact")
+
+    To link an artifact from a previous run you can use ``use_artifact`` function:
+
+    .. code-block:: python
+
+        from pytorch_lightning.loggers import WandbLogger
+
+        wandb_logger = WandbLogger(project="my_project", name="my_run")
+        wandb_logger.use_artifact(artifact="path/to/artifact")
+
     See Also:
         - `Demo in Google Colab <http://wandb.me/lightning>`__ with hyperparameter search and model logging
         - `W&B Documentation <https://docs.wandb.ai/integrations/lightning>`__
@@ -224,17 +261,18 @@ class WandbLogger(Logger):
     Args:
         name: Display name for the run.
         save_dir: Path where data is saved.
+        version: Sets the version, mainly used to resume a previous run.
         offline: Run offline (data can be streamed later to wandb servers).
-        id: Sets the version, mainly used to resume a previous run.
-        version: Same as id.
+        dir: Same as save_dir.
+        id: Same as version.
         anonymous: Enables or explicitly disables anonymous logging.
         project: The name of the project to which this run will belong.
-        log_model: Log checkpoints created by :class:`~pytorch_lightning.callbacks.model_checkpoint.ModelCheckpoint`
+        log_model: Log checkpoints created by :class:`~pytorch_lightning.callbacks.ModelCheckpoint`
             as W&B artifacts. `latest` and `best` aliases are automatically set.
 
             * if ``log_model == 'all'``, checkpoints are logged during training.
             * if ``log_model == True``, checkpoints are logged at the end of training, except when
-              :paramref:`~pytorch_lightning.callbacks.model_checkpoint.ModelCheckpoint.save_top_k` ``== -1``
+              :paramref:`~pytorch_lightning.callbacks.ModelCheckpoint.save_top_k` ``== -1``
               which also logs every checkpoint during training.
             * if ``log_model == False`` (default), no checkpoint is logged.
 
@@ -255,17 +293,16 @@ class WandbLogger(Logger):
     def __init__(
         self,
         name: Optional[str] = None,
-        save_dir: str = ".",
+        save_dir: _PATH = ".",
+        version: Optional[str] = None,
         offline: bool = False,
+        dir: Optional[_PATH] = None,
         id: Optional[str] = None,
         anonymous: Optional[bool] = None,
-        version: Optional[str] = None,
         project: str = "lightning_logs",
         log_model: Union[str, bool] = False,
         experiment: Union[Run, RunDisabled, None] = None,
         prefix: str = "",
-        agg_key_funcs: Optional[Mapping[str, Callable[[Sequence[float]], float]]] = None,
-        agg_default_func: Optional[Callable[[Sequence[float]], float]] = None,
         **kwargs: Any,
     ) -> None:
         if wandb is None:
@@ -288,19 +325,26 @@ class WandbLogger(Logger):
                 "Hint: Upgrade with `pip install --upgrade wandb`."
             )
 
-        super().__init__(agg_key_funcs=agg_key_funcs, agg_default_func=agg_default_func)
+        super().__init__()
         self._offline = offline
         self._log_model = log_model
         self._prefix = prefix
         self._experiment = experiment
         self._logged_model_time: Dict[str, float] = {}
-        self._checkpoint_callback: Optional["ReferenceType[Checkpoint]"] = None
+        self._checkpoint_callback: Optional[Checkpoint] = None
+
+        # paths are processed as strings
+        if save_dir is not None:
+            save_dir = os.fspath(save_dir)
+        elif dir is not None:
+            dir = os.fspath(dir)
+
         # set wandb init arguments
         self._wandb_init: Dict[str, Any] = dict(
             name=name,
             project=project,
+            dir=save_dir or dir,
             id=version or id,
-            dir=save_dir or kwargs.pop("dir"),
             resume="allow",
             anonymous=("allow" if anonymous else None),
         )
@@ -327,7 +371,7 @@ class WandbLogger(Logger):
         state["_experiment"] = None
         return state
 
-    @property  # type: ignore[misc]
+    @property
     @rank_zero_experiment
     def experiment(self) -> Union[Run, RunDisabled]:
         r"""
@@ -469,7 +513,7 @@ class WandbLogger(Logger):
         # don't create an experiment if we don't have one
         return self._experiment.id if self._experiment else self._id
 
-    def after_save_checkpoint(self, checkpoint_callback: "ReferenceType[Checkpoint]") -> None:
+    def after_save_checkpoint(self, checkpoint_callback: Checkpoint) -> None:
         # log checkpoints as artifacts
         if (
             self._log_model == "all"
@@ -481,37 +525,64 @@ class WandbLogger(Logger):
         elif self._log_model is True:
             self._checkpoint_callback = checkpoint_callback
 
+    @staticmethod
+    @rank_zero_only
+    def download_artifact(
+        artifact: str,
+        save_dir: Optional[_PATH] = None,
+        artifact_type: Optional[str] = None,
+        use_artifact: Optional[bool] = True,
+    ) -> str:
+        """Downloads an artifact from the wandb server.
+
+        Args:
+            artifact: The path of the artifact to download.
+            save_dir: The directory to save the artifact to.
+            artifact_type: The type of artifact to download.
+            use_artifact: Whether to add an edge between the artifact graph.
+
+        Returns:
+            The path to the downloaded artifact.
+        """
+        if wandb.run is not None and use_artifact:
+            artifact = wandb.run.use_artifact(artifact)
+        else:
+            api = wandb.Api()
+            artifact = api.artifact(artifact, type=artifact_type)
+
+        save_dir = None if save_dir is None else os.fspath(save_dir)
+        return artifact.download(root=save_dir)
+
+    def use_artifact(self, artifact: str, artifact_type: Optional[str] = None) -> "wandb.Artifact":
+        """Logs to the wandb dashboard that the mentioned artifact is used by the run.
+
+        Args:
+            artifact: The path of the artifact.
+            artifact_type: The type of artifact being used.
+
+        Returns:
+            wandb Artifact object for the artifact.
+        """
+        return self.experiment.use_artifact(artifact, type=artifact_type)
+
     @rank_zero_only
     def finalize(self, status: str) -> None:
+        if status != "success":
+            # Currently, checkpoints only get logged on success
+            return
         # log checkpoints as artifacts
-        if self._checkpoint_callback:
+        if self._checkpoint_callback and self._experiment is not None:
             self._scan_and_log_checkpoints(self._checkpoint_callback)
 
-    def _scan_and_log_checkpoints(self, checkpoint_callback: "ReferenceType[Checkpoint]") -> None:
+    def _scan_and_log_checkpoints(self, checkpoint_callback: Checkpoint) -> None:
         # get checkpoints to be saved with associated score
-        checkpoints = dict()
-        if hasattr(checkpoint_callback, "last_model_path") and hasattr(checkpoint_callback, "current_score"):
-            checkpoints[checkpoint_callback.last_model_path] = (checkpoint_callback.current_score, "latest")
-
-        if hasattr(checkpoint_callback, "best_model_path") and hasattr(checkpoint_callback, "best_model_score"):
-            checkpoints[checkpoint_callback.best_model_path] = (checkpoint_callback.best_model_score, "best")
-
-        if hasattr(checkpoint_callback, "best_k_models"):
-            for key, value in checkpoint_callback.best_k_models.items():
-                checkpoints[key] = (value, "best_k")
-
-        checkpoints = sorted(
-            (Path(p).stat().st_mtime, p, s, tag) for p, (s, tag) in checkpoints.items() if Path(p).is_file()
-        )
-        checkpoints = [
-            c for c in checkpoints if c[1] not in self._logged_model_time.keys() or self._logged_model_time[c[1]] < c[0]
-        ]
+        checkpoints = _scan_checkpoints(checkpoint_callback, self._logged_model_time)
 
         # log iteratively all new checkpoints
         for t, p, s, tag in checkpoints:
             metadata = (
                 {
-                    "score": s,
+                    "score": s.item() if isinstance(s, Tensor) else s,
                     "original_filename": Path(p).name,
                     checkpoint_callback.__class__.__name__: {
                         k: getattr(checkpoint_callback, k)
