@@ -11,16 +11,13 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-import functools
 import inspect
-import os
-from collections import OrderedDict
-from contextlib import contextmanager
 from dataclasses import fields
-from functools import partial
-from typing import Any, Callable, Dict, Generator, Iterable, Mapping, Optional, Set, Tuple, Type, Union
+from typing import Any, Dict, Generator, Iterable, Mapping, Optional, Tuple, Union
 
 import torch
+from lightning_utilities.core.apply_func import is_dataclass_instance
+from lightning_utilities.core.rank_zero import WarningCache
 from torch import Tensor
 from torch.utils.data import (
     BatchSampler,
@@ -33,34 +30,24 @@ from torch.utils.data import (
 )
 
 import pytorch_lightning as pl
+from lightning_lite.utilities.data import _reinstantiate_wrapped_cls, _replace_value_in_saved_args
+from lightning_lite.utilities.data import has_iterable_dataset as new_has_iterable_dataset
+from lightning_lite.utilities.data import has_len as new_has_len
 from pytorch_lightning.overrides.distributed import IndexBatchSamplerWrapper
 from pytorch_lightning.trainer.states import RunningStage
-from pytorch_lightning.utilities.apply_func import _is_dataclass_instance
+from pytorch_lightning.trainer.supporters import CombinedLoader
 from pytorch_lightning.utilities.auto_restart import CaptureIterableDataset, CaptureMapDataset, FastForwardSampler
-from pytorch_lightning.utilities.enums import _FaultTolerantMode, LightningEnum
+from pytorch_lightning.utilities.enums import _FaultTolerantMode
 from pytorch_lightning.utilities.exceptions import MisconfigurationException
-from pytorch_lightning.utilities.rank_zero import rank_zero_warn
-from pytorch_lightning.utilities.seed import pl_worker_init_function
-from pytorch_lightning.utilities.warnings import WarningCache
+from pytorch_lightning.utilities.rank_zero import rank_zero_deprecation, rank_zero_warn
 
-BType = Union[Tensor, str, Mapping[Any, "BType"], Iterable["BType"]]
+# might be supported in later releases, see https://github.com/python/mypy/pull/13297
+BType = Union[Tensor, str, Mapping[Any, "BType"], Iterable["BType"]]  # type: ignore[misc]
 
 warning_cache = WarningCache()
 
 
-class _WrapAttrTag(LightningEnum):
-    SET = "set"
-    DEL = "del"
-
-    def __call__(self, *args):
-        if self == self.SET:
-            fn = setattr
-        else:
-            fn = delattr
-        return fn(*args)
-
-
-def _extract_batch_size(batch: BType) -> Generator[int, None, None]:
+def _extract_batch_size(batch: BType) -> Generator[Optional[int], None, None]:
     if isinstance(batch, Tensor):
         if batch.ndim == 0:
             yield 1
@@ -72,7 +59,7 @@ def _extract_batch_size(batch: BType) -> Generator[int, None, None]:
 
         for sample in batch:
             yield from _extract_batch_size(sample)
-    elif _is_dataclass_instance(batch):
+    elif is_dataclass_instance(batch):
         for field in fields(batch):
             yield from _extract_batch_size(getattr(batch, field.name))
     else:
@@ -109,45 +96,16 @@ def extract_batch_size(batch: BType) -> int:
     return batch_size
 
 
-def has_iterable_dataset(dataloader: DataLoader) -> bool:
-    return hasattr(dataloader, "dataset") and isinstance(dataloader.dataset, IterableDataset)
-
-
-def has_len(dataloader: Union[DataLoader, Iterable]) -> bool:
-    """Checks if a given Dataloader has ``__len__`` method implemented i.e. if it is a finite dataloader or
-    infinite dataloader."""
-    try:
-        # try getting the length
-        if len(dataloader) == 0:
-            rank_zero_warn(
-                f"`{dataloader.__class__.__name__}` returned 0 length. Please make sure this was your intention."
-            )
-        has_len = True
-    except TypeError:
-        has_len = False
-    except NotImplementedError:  # e.g. raised by torchtext if a batch_size_fn is used
-        has_len = False
-
-    if has_len and has_iterable_dataset(dataloader):
-        rank_zero_warn(
-            "Your `IterableDataset` has `__len__` defined."
-            " In combination with multi-process data loading (when num_workers > 1),"
-            " `__len__` could be inaccurate if each worker is not configured independently"
-            " to avoid having duplicate data."
-        )
-    return has_len
-
-
 def has_len_all_ranks(
-    dataloader: DataLoader,
-    training_type: "pl.Strategy",
+    dataloader: Union[DataLoader, CombinedLoader],
+    strategy: "pl.strategies.Strategy",
     model: Union["pl.LightningModule", "pl.LightningDataModule"],
 ) -> bool:
     """Checks if a given Dataloader has ``__len__`` method implemented i.e. if it is a finite dataloader or
     infinite dataloader."""
     try:
-        local_length = len(dataloader)
-        total_length = training_type.reduce(torch.tensor(local_length).to(model.device), reduce_op="sum")
+        local_length = len(dataloader)  # type: ignore [arg-type] # we are checking with duck-typing
+        total_length = strategy.reduce(torch.tensor(local_length, device=strategy.root_device), reduce_op="sum")
 
         if total_length == 0:
             rank_zero_warn(
@@ -169,12 +127,11 @@ def has_len_all_ranks(
         else:
             has_len = True
 
-    except TypeError:
-        has_len = False
-    except NotImplementedError:  # e.g. raised by torchtext if a batch_size_fn is used
+    except (TypeError, NotImplementedError):
         has_len = False
 
-    if has_len and has_iterable_dataset(dataloader):
+    # we are checking using lightning_lite, which doesn't know CombinedLoader
+    if has_len and new_has_iterable_dataset(dataloader):  # type: ignore [arg-type]
         rank_zero_warn(
             "Your `IterableDataset` has `__len__` defined."
             " In combination with multi-process data loading (when num_workers > 1),"
@@ -184,14 +141,14 @@ def has_len_all_ranks(
     return has_len
 
 
-def get_len(dataloader: DataLoader) -> Union[int, float]:
+def get_len(dataloader: Union[DataLoader, Dataset]) -> Union[int, float]:
     """Return the length of the given DataLoader.
 
     If ``__len__`` method is not implemented, return float('inf').
     """
 
-    if has_len(dataloader):
-        return len(dataloader)
+    if new_has_len(dataloader):
+        return len(dataloader)  # type: ignore [arg-type]
 
     return float("inf")
 
@@ -206,7 +163,7 @@ def _update_dataloader(
 
 def _get_dataloader_init_args_and_kwargs(
     dataloader: DataLoader,
-    sampler: Optional[Sampler],
+    sampler: Union[Sampler, Iterable],
     mode: Optional[RunningStage] = None,
     disallow_batch_sampler: bool = False,
 ) -> Tuple[Tuple[Any], Dict[str, Any]]:
@@ -230,7 +187,7 @@ def _get_dataloader_init_args_and_kwargs(
         arg_names = ()
 
     # get the dataloader instance `__init__` parameters
-    params = dict(inspect.signature(dataloader.__init__).parameters)
+    params = dict(inspect.signature(dataloader.__init__).parameters)  # type: ignore[misc]
     has_variadic_kwargs = any(p.kind is p.VAR_KEYWORD for p in params.values())
     if has_variadic_kwargs:
         # if the signature takes **kwargs, assume they will be passed down with `super().__init__(**kwargs)`
@@ -247,7 +204,7 @@ def _get_dataloader_init_args_and_kwargs(
 
     if not was_wrapped:
         # keep only the params whose default is different to the current attr value
-        non_defaults = {name for name, p in params.items() if name in attrs and p.default != attrs[name]}
+        non_defaults = {name for name, p in params.items() if name in attrs and p.default is not attrs[name]}
 
         # add `dataset` as it might have been replaced with `*args`
         non_defaults.add("dataset")
@@ -272,14 +229,14 @@ def _get_dataloader_init_args_and_kwargs(
     }
     # the dataloader has required args which we could not extract from the existing attributes
     if required_args:
-        required_args = sorted(required_args)
+        sorted_required_args = sorted(required_args)
         dataloader_cls_name = dataloader.__class__.__name__
-        missing_args_message = ", ".join(f"`self.{arg_name}`" for arg_name in required_args)
+        missing_args_message = ", ".join(f"`self.{arg_name}`" for arg_name in sorted_required_args)
         raise MisconfigurationException(
             f"Trying to inject custom `Sampler` into the `{dataloader_cls_name}` instance. "
             "This would fail as some of the `__init__` arguments are not available as instance attributes. "
-            f"The missing attributes are {required_args}. If you instantiate your `{dataloader_cls_name}` inside a "
-            "`*_dataloader` hook of your module, we will do this for you."
+            f"The missing attributes are {sorted_required_args}. If you instantiate your `{dataloader_cls_name}` "
+            "inside a `*_dataloader` hook of your module, we will do this for you."
             f" Otherwise, define {missing_args_message} inside your `__init__`."
         )
 
@@ -287,13 +244,13 @@ def _get_dataloader_init_args_and_kwargs(
         # the dataloader signature does not allow keyword arguments that need to be passed
         missing_kwargs = (set(dl_kwargs) | set(arg_names)) - params.keys()
         if missing_kwargs:
-            missing_kwargs = sorted(missing_kwargs)
+            sorted_missing_kwargs = sorted(missing_kwargs)
             dataloader_cls_name = dataloader.__class__.__name__
             raise MisconfigurationException(
                 f"Trying to inject parameters into the `{dataloader_cls_name}` instance. "
                 "This would fail as it doesn't expose all its attributes in the `__init__` signature. "
-                f"The missing arguments are {missing_kwargs}. HINT: If you wrote the `{dataloader_cls_name}` class, "
-                "add the `__init__` arguments or allow passing `**kwargs`"
+                f"The missing arguments are {sorted_missing_kwargs}. HINT: If you wrote the `{dataloader_cls_name}` "
+                "class, add the `__init__` arguments or allow passing `**kwargs`"
             )
 
     if _FaultTolerantMode.detect_current_mode().is_automatic:
@@ -306,7 +263,7 @@ def _get_dataloader_init_args_and_kwargs(
 
 def _dataloader_init_kwargs_resolve_sampler(
     dataloader: DataLoader,
-    sampler: Optional[Sampler],
+    sampler: Union[Sampler, Iterable],
     mode: Optional[RunningStage] = None,
     disallow_batch_sampler: bool = False,
 ) -> Dict[str, Any]:
@@ -412,185 +369,6 @@ def _dataloader_init_kwargs_resolve_sampler(
     return {"sampler": sampler, "shuffle": False, "batch_sampler": None}
 
 
-def _replace_value_in_saved_args(
-    replace_key: str,
-    replace_value: Any,
-    args: Tuple[Any, ...],
-    kwargs: Dict[str, Any],
-    default_kwargs: Dict[str, Any],
-    arg_names: Tuple[str, ...],
-) -> Tuple[bool, Tuple[Any, ...], Dict[str, Any]]:
-    """Tries to replace an argument value in a saved list of args and kwargs.
-
-    Returns a tuple indicating success of the operation and modified saved args and kwargs
-    """
-
-    if replace_key in arg_names:
-        replace_index = arg_names.index(replace_key)
-        args = args[:replace_index] + (replace_value,) + args[replace_index + 1 :]
-        return True, args, kwargs
-    elif replace_key in kwargs or replace_key in default_kwargs:
-        kwargs[replace_key] = replace_value
-        return True, args, kwargs
-
-    return False, args, kwargs
-
-
-def _auto_add_worker_init_fn(dataloader: DataLoader, rank: int) -> None:
-    if int(os.environ.get("PL_SEED_WORKERS", 0)) and dataloader.worker_init_fn is None:
-        dataloader.worker_init_fn = partial(pl_worker_init_function, rank=rank)
-
-
-def _reinstantiate_wrapped_cls(orig_object: Any, *args: Any, explicit_cls: Optional[Type] = None, **kwargs: Any) -> Any:
-    constructor = type(orig_object) if explicit_cls is None else explicit_cls
-
-    try:
-        result = constructor(*args, **kwargs)
-    except TypeError as e:
-        # improve exception message due to an incorrect implementation of the `DataLoader` where multiple subclass
-        # `__init__` arguments map to one `DataLoader.__init__` argument
-        import re
-
-        match = re.match(r".*__init__\(\) got multiple values .* '(\w+)'", str(e))
-        if not match:
-            # an unexpected `TypeError`, continue failure
-            raise
-        argument = match.groups()[0]
-        message = (
-            f"The {constructor.__name__} implementation has an error where more than one `__init__` argument"
-            f" can be passed to its parent's `{argument}=...` `__init__` argument. This is likely caused by allowing"
-            f" passing both a custom argument that will map to the `{argument}` argument as well as `**kwargs`."
-            f" `kwargs` should be filtered to make sure they don't contain the `{argument}` key."
-            " This argument was automatically passed to your object by PyTorch Lightning."
-        )
-        raise MisconfigurationException(message) from e
-
-    attrs_record = getattr(orig_object, "__pl_attrs_record", list())
-    for args, fn in attrs_record:
-        fn(result, *args)
-
-    return result
-
-
-def _wrap_init_method(init: Callable, store_explicit_arg: Optional[str] = None) -> Callable:
-    """Wraps the ``__init__`` method of classes (currently :class:`~torch.utils.data.DataLoader` and
-    :class:`~torch.utils.data.BatchSampler`) in order to enable re-instantiation of custom subclasses."""
-
-    @functools.wraps(init)
-    def wrapper(obj: Any, *args: Any, **kwargs: Any) -> None:
-        # We need to inspect `init`, as inspecting `obj.__init__`
-        # can lead to inspecting the wrong function with multiple inheritance
-        old_inside_init = getattr(obj, "__pl_inside_init", False)
-        object.__setattr__(obj, "__pl_inside_init", True)
-        params = inspect.signature(init).parameters
-
-        parameters_defaults = OrderedDict(
-            (param.name, param.default)
-            for param in params.values()
-            if param.name != "self" and param.kind not in (param.VAR_POSITIONAL, param.VAR_KEYWORD)
-        )
-
-        param_names = tuple(parameters_defaults)[: len(args)]
-
-        default_kwargs = {
-            name: value
-            for name, value in parameters_defaults.items()
-            if name not in kwargs and name not in param_names and value != inspect.Parameter.empty
-        }
-
-        if not hasattr(obj, "__pl_saved_args"):
-            object.__setattr__(obj, "__pl_saved_args", args)
-            object.__setattr__(obj, "__pl_saved_kwargs", kwargs)
-            object.__setattr__(obj, "__pl_saved_arg_names", param_names)
-            object.__setattr__(obj, "__pl_saved_default_kwargs", default_kwargs)
-
-        # We want to use the latest possible value for explicit argument (i.e. ideally what gets passed to base class)
-        # so that we can be sure, that it will not get changed anymore.
-        # That is why we are setting this in every `__init__`
-        if store_explicit_arg is not None:
-            if store_explicit_arg in param_names:
-                object.__setattr__(obj, f"__{store_explicit_arg}", args[param_names.index(store_explicit_arg)])
-            elif store_explicit_arg in kwargs:
-                object.__setattr__(obj, f"__{store_explicit_arg}", kwargs[store_explicit_arg])
-
-        init(obj, *args, **kwargs)
-        object.__setattr__(obj, "__pl_inside_init", old_inside_init)
-
-    return wrapper
-
-
-def _wrap_attr_method(method: Callable, tag: _WrapAttrTag) -> Callable:
-    """Wraps the ``__setattr__`` or ``__delattr__`` method of classes (currently :class:`~torch.utils.data.DataLoader` and
-    :class:`~torch.utils.data.BatchSampler`) in order to enable re-instantiation of custom subclasses."""
-
-    @functools.wraps(method)
-    def wrapper(obj: Any, *args: Any):
-        # First, let's find out if we're the first in inheritance chain calling the patched method.
-        name, *_ = args
-        prev_call_name, prev_call_method = getattr(obj, "__pl_current_call", (None, "method"))
-        first_call = not (prev_call_name == name and prev_call_method == tag)
-
-        # Then mark the current called method
-        object.__setattr__(obj, "__pl_current_call", (name, tag))
-
-        # call original method
-        method(obj, *args)
-        if first_call and not getattr(obj, "__pl_inside_init", True):
-            # and save the value it was called with to the internal list,
-            # if we're outside of __init__ and the original call did not fail and we're the first call
-            attrs_record = getattr(obj, "__pl_attrs_record", list())
-            attrs_record.append((args, tag))
-            object.__setattr__(obj, "__pl_attrs_record", attrs_record)
-        object.__setattr__(obj, "__pl_current_call", (prev_call_name, prev_call_method))
-
-    return wrapper
-
-
-# https://stackoverflow.com/a/63851681/9201239
-def _get_all_subclasses(cls: Type[Any]) -> Set[Type[Any]]:
-    """Returns a list of all classes that inherit directly or indirectly from the given class."""
-    subclasses = set()
-
-    def recurse(cl: Type[Any]) -> None:
-        for subclass in cl.__subclasses__():
-            subclasses.add(subclass)
-            recurse(subclass)
-
-    recurse(cls)
-    return subclasses
-
-
-@contextmanager
-def _replace_dunder_methods(base_cls: Type, store_explicit_arg: Optional[str] = None) -> Generator[None, None, None]:
-    """This context manager is used to add support for re-instantiation of custom (subclasses) of `base_cls`.
-
-    It patches the ``__init__``, ``__setattr__`` and ``__delattr__`` methods.
-    """
-    classes = _get_all_subclasses(base_cls) | {base_cls}
-    for cls in classes:
-        # Check that __init__ belongs to the class
-        # https://stackoverflow.com/a/5253424
-        if "__init__" in cls.__dict__:
-            cls.__old__init__ = cls.__init__
-            cls.__init__ = _wrap_init_method(cls.__init__, store_explicit_arg)
-
-        # we want at least one setattr/delattr in the chain to be patched and it can happen, that none of the subclasses
-        # implement `__setattr__`/`__delattr__`. Therefore, we are always patching the `base_cls`
-        for patch_fn_name, tag in (("__setattr__", _WrapAttrTag.SET), ("__delattr__", _WrapAttrTag.DEL)):
-            if patch_fn_name in cls.__dict__ or cls is base_cls:
-                saved_name = f"__old{patch_fn_name}"
-                setattr(cls, saved_name, getattr(cls, patch_fn_name))
-                setattr(cls, patch_fn_name, _wrap_attr_method(getattr(cls, patch_fn_name), tag))
-    yield
-    for cls in classes:
-        for patched_name in ("__setattr__", "__delattr__", "__init__"):
-            # Check that __old__{init,setattr,delattr} belongs to the class
-            # https://stackoverflow.com/a/5253424
-            if f"__old{patched_name}" in cls.__dict__:
-                setattr(cls, patched_name, getattr(cls, f"__old{patched_name}"))
-                delattr(cls, f"__old{patched_name}")
-
-
 def _wrap_with_capture_dataset(dataset: Dataset) -> Dataset:
     if isinstance(dataset, IterableDataset):
         # wrap the `IterableDataset` into a `CaptureIterableDataset` to record sampler states.
@@ -644,3 +422,19 @@ def _is_dataloader_shuffled(dataloader: object) -> bool:
     if isinstance(sampler, SequentialSampler):
         return False
     return isinstance(sampler, RandomSampler)
+
+
+def has_iterable_dataset(*args: Any, **kwargs: Any) -> Any:
+    rank_zero_deprecation(
+        "`pytorch_lightning.utilities.data.has_iterable_dataset` has been deprecated in v1.8.0 and will be"
+        " removed in v1.10.0. Please use `lightning_lite.utilities.data.has_iterable_dataset` instead."
+    )
+    return new_has_iterable_dataset(*args, **kwargs)
+
+
+def has_len(*args: Any, **kwargs: Any) -> Any:
+    rank_zero_deprecation(
+        "`pytorch_lightning.utilities.data.has_len` has been deprecated in v1.8.0 and will be"
+        " removed in v1.10.0. Please use `lightning_lite.utilities.data.has_len` instead."
+    )
+    return new_has_len(*args, **kwargs)
