@@ -16,22 +16,18 @@ Convention:
  - Do not include any `_TYPE` suffix
  - Types used in public hooks (as those in the `LightningModule` and `Callback`) should be public (no leading `_`)
 """
+from argparse import _ArgumentGroup, ArgumentParser
 from contextlib import contextmanager
 from dataclasses import dataclass
-from pathlib import Path
-from typing import Any, Dict, Generator, Iterator, List, Mapping, Optional, Sequence, Type, Union
+from typing import Any, Dict, Generator, List, Mapping, Optional, Sequence, Type, Union
 
 import torch
 from torch import Tensor
-from torch.optim import Optimizer
 from torch.utils.data import DataLoader
 from torchmetrics import Metric
 from typing_extensions import Protocol, runtime_checkable
 
-if torch.distributed.is_available():
-    from torch._C._distributed_c10d import ProcessGroup
-else:
-    ProcessGroup = Any  # type: ignore[assignment,misc]
+from lightning_lite.utilities.types import _LRScheduler, ProcessGroup, ReduceLROnPlateau
 
 _NUMBER = Union[int, float]
 _METRIC = Union[Metric, Tensor, _NUMBER]
@@ -40,8 +36,6 @@ STEP_OUTPUT = Union[Tensor, Dict[str, Any]]
 EPOCH_OUTPUT = List[STEP_OUTPUT]
 _EVALUATE_OUTPUT = List[Dict[str, float]]  # 1 dict per DataLoader
 _PREDICT_OUTPUT = Union[List[Any], List[List[Any]]]
-_PARAMETERS = Iterator[torch.nn.Parameter]
-_PATH = Union[str, Path]
 TRAIN_DATALOADERS = Union[
     DataLoader,
     Sequence[DataLoader],
@@ -52,7 +46,7 @@ TRAIN_DATALOADERS = Union[
     Dict[str, Sequence[DataLoader]],
 ]
 EVAL_DATALOADERS = Union[DataLoader, Sequence[DataLoader]]
-_DEVICE = Union[torch.device, str, int]
+_ADD_ARGPARSE_RETURN = Union[_ArgumentGroup, ArgumentParser]
 
 
 @runtime_checkable
@@ -91,57 +85,6 @@ class PredictStep(Protocol):
         ...
 
 
-@runtime_checkable
-class _Stateful(Protocol):
-    """This class is used to detect if an object is stateful using `isinstance(obj, _Stateful)`."""
-
-    def state_dict(self) -> Dict[str, Any]:
-        ...
-
-    def load_state_dict(self, state_dict: Dict[str, Any]) -> None:
-        ...
-
-
-# Inferred from `torch.optim.lr_scheduler.pyi`
-# Missing attributes were added to improve typing
-@runtime_checkable
-class _LRScheduler(_Stateful, Protocol):
-    optimizer: Optimizer
-    base_lrs: List[float]
-
-    def __init__(self, optimizer: Optimizer, *args: Any, **kwargs: Any) -> None:
-        ...
-
-    def step(self, epoch: Optional[int] = None) -> None:
-        ...
-
-
-# Inferred from `torch.optim.lr_scheduler.pyi`
-# Missing attributes were added to improve typing
-@runtime_checkable
-class ReduceLROnPlateau(_Stateful, Protocol):
-    in_cooldown: bool
-    optimizer: Optimizer
-
-    def __init__(
-        self,
-        optimizer: Optimizer,
-        mode: str = ...,
-        factor: float = ...,
-        patience: int = ...,
-        verbose: bool = ...,
-        threshold: float = ...,
-        threshold_mode: str = ...,
-        cooldown: int = ...,
-        min_lr: float = ...,
-        eps: float = ...,
-    ) -> None:
-        ...
-
-    def step(self, metrics: Union[float, int, Tensor], epoch: Optional[int] = None) -> None:
-        ...
-
-
 # Inferred from `torch.nn.parallel.distributed.pyi`
 # Missing attributes were added to improve typing
 @runtime_checkable
@@ -171,6 +114,7 @@ class DistributedDataParallel(Protocol):
 LRSchedulerTypeTuple = (torch.optim.lr_scheduler._LRScheduler, torch.optim.lr_scheduler.ReduceLROnPlateau)
 LRSchedulerTypeUnion = Union[torch.optim.lr_scheduler._LRScheduler, torch.optim.lr_scheduler.ReduceLROnPlateau]
 LRSchedulerType = Union[Type[torch.optim.lr_scheduler._LRScheduler], Type[torch.optim.lr_scheduler.ReduceLROnPlateau]]
+LRSchedulerPLType = Union[_LRScheduler, ReduceLROnPlateau]
 
 
 @dataclass
